@@ -22,7 +22,7 @@
  * @author Samuel Mimram
  */
 
-/* $Id: alsa_stubs.c 4648 2007-10-06 16:45:26Z dbaelde $ */
+/* $Id$ */
 
 #include <string.h>
 #include <alsa/asoundlib.h>
@@ -64,6 +64,7 @@ static struct custom_operations pcm_handle_ops =
 #define Frame_size_val(v) Pcm_val(v)->frame_size
 
 #define int_of_direction(dir) (Int_val(dir)-1)
+#define direction_of_int(dir) (Val_int(dir+1))
 
 static int int_of_pcm_stream(value stream)
 {
@@ -114,7 +115,11 @@ static int int_of_pcm_mode(value mode)
   return ans;
 }
 
-static void check_for_err(int ret)
+/* Here, we use the type snd_pcm_sframes_t
+ * which is long usually and makes sure we
+ * avoid long to int overflow in the read/write
+ * function's return value. */
+static void check_for_err(snd_pcm_sframes_t ret)
 {
   if (ret >= 0)
     return;
@@ -124,20 +129,37 @@ static void check_for_err(int ret)
 
   switch(-ret)
   {
-    case 5:
+    case EIO:
       caml_raise_constant(*caml_named_value("alsa_exn_io_error"));
       break;
 
-    case 16:
+    case EBUSY:
       caml_raise_constant(*caml_named_value("alsa_exn_device_busy"));
       break;
 
-    case 22:
+    case EINVAL:
       caml_raise_constant(*caml_named_value("alsa_exn_invalid_argument"));
       break;
 
-    case 32:
-      caml_raise_constant(*caml_named_value("alsa_exn_broken_pipe"));
+    case EPIPE:
+      caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
+      break;
+
+    case ESTRPIPE:
+      caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
+      break;
+
+    case EBADFD:
+      caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
+      break;
+
+    case EINTR:
+      caml_raise_constant(*caml_named_value("alsa_exn_interrupted"));
+      break;
+
+    case ENOTTY:
+    case ENODEV:
+      caml_raise_constant(*caml_named_value("alsa_exn_device_busy"));
       break;
 
     default:
@@ -146,6 +168,30 @@ static void check_for_err(int ret)
   }
 }
 
+CAMLprim value ocaml_snd_int_of_error(value name)
+{
+  CAMLparam1(name);
+  char *s = String_val(name);
+  if (!strcmp(s,"alsa_exn_io_error"))
+    CAMLreturn(Val_int(-EIO));
+  if (!strcmp(s,"alsa_exn_device_busy"))
+    CAMLreturn(Val_int(-EBUSY));
+  if (!strcmp(s,"alsa_exn_invalid_argument"))
+    CAMLreturn(Val_int(-EINVAL));
+  if (!strcmp(s,"alsa_exn_buffer_xrun"))
+    CAMLreturn(Val_int(-EPIPE));
+  if (!strcmp(s,"alsa_exn_suspended"))
+    CAMLreturn(Val_int(-ESTRPIPE));
+  if (!strcmp(s,"alsa_exn_bad_state"))
+    CAMLreturn(Val_int(-EBADFD));
+  if (!strcmp(s,"alsa_exn_interrupted"))
+    CAMLreturn(Val_int(-EINTR));
+  if (!strcmp(s,"alsa_exn_device_busy"))
+    CAMLreturn(Val_int(-ENODEV));
+ 
+  caml_failwith("unknown value");
+}
+
 CAMLprim value ocaml_snd_string_of_error(value n)
 {
   CAMLparam1(n);
@@ -196,7 +242,7 @@ CAMLprim value ocaml_snd_pcm_open(value name, value stream, value mode)
 {
   CAMLparam3(name, stream, mode);
   CAMLlocal1(ans);
-  int ret;
+  snd_pcm_sframes_t ret;
   my_snd_pcm_t *hnd;
 
   ans = caml_alloc_custom(&pcm_handle_ops, sizeof(my_snd_pcm_t), 0, 1);
@@ -224,6 +270,20 @@ CAMLprim value ocaml_snd_pcm_prepare(value handle)
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value ocaml_snd_pcm_resume(value handle)
+{
+  CAMLparam1(handle);
+  check_for_err(snd_pcm_resume(Pcm_handle_val(handle)));
+  CAMLreturn(Val_unit);
+}
+
+CAMLprim value ocaml_snd_pcm_recover(value handle, value err, value log)
+{
+  CAMLparam1(handle);
+  check_for_err(snd_pcm_recover(Pcm_handle_val(handle),Int_val(err),Bool_val(log)));
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value ocaml_snd_pcm_wait(value handle, value timeout)
 {
   CAMLparam2(handle, timeout);
@@ -241,7 +301,7 @@ CAMLprim value ocaml_snd_pcm_readi(value handle_, value dbuf, value ofs_, value
   int ofs = Int_val(ofs_);
   snd_pcm_t *handle = Pcm_handle_val(handle_);
   char *buf;
-  int ret;
+  snd_pcm_sframes_t ret;
 
   if (ofs + len * Frame_size_val(handle_) > caml_string_length(dbuf))
     caml_invalid_argument("buffer");
@@ -253,12 +313,6 @@ CAMLprim value ocaml_snd_pcm_readi(value handle_, value dbuf, value ofs_, value
 
   memcpy(String_val(dbuf) + ofs, buf, len * Frame_size_val(handle_));
   free(buf);
-  if (ret == -EPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
-  else if (ret == -EBADFD)
-    caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
-  else if (ret == -ESTRPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
   check_for_err(ret);
 
   CAMLreturn(Val_int(ret));
@@ -271,7 +325,7 @@ CAMLprim value ocaml_snd_pcm_writei(value handle_, value sbuf, value ofs_, value
   int ofs = Int_val(ofs_);
   snd_pcm_t *handle = Pcm_handle_val(handle_);
   char *buf;
-  int ret;
+  snd_pcm_sframes_t ret;
 
   if (ofs + len * Frame_size_val(handle_) > caml_string_length(sbuf))
     caml_invalid_argument("buffer");
@@ -283,12 +337,6 @@ CAMLprim value ocaml_snd_pcm_writei(value handle_, value sbuf, value ofs_, value
   caml_leave_blocking_section();
 
   free(buf);
-  if (ret == -EPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
-  else if (ret == -EBADFD)
-    caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
-  else if (ret == -ESTRPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
   check_for_err(ret);
 
   CAMLreturn(Val_int(ret));
@@ -303,7 +351,7 @@ CAMLprim value ocaml_snd_pcm_readn(value handle_, value dbuf, value ofs_, value
   snd_pcm_t *handle = Pcm_handle_val(handle_);
   char **buf;
   int c;
-  int ret;
+  snd_pcm_sframes_t ret;
 
   /* TODO: check the size of dbuf */
   buf = malloc(chans * sizeof(char*));
@@ -320,12 +368,6 @@ CAMLprim value ocaml_snd_pcm_readn(value handle_, value dbuf, value ofs_, value
     free(buf[c]);
   }
   free(buf);
-  if (ret == -EPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
-  else if (ret == -EBADFD)
-    caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
-  else if (ret == -ESTRPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
   check_for_err(ret);
 
   CAMLreturn(Val_int(ret));
@@ -340,7 +382,7 @@ CAMLprim value ocaml_snd_pcm_writen(value handle_, value sbuf, value ofs_, value
   snd_pcm_t *handle = Pcm_handle_val(handle_);
   char **buf;
   int c;
-  int ret;
+  snd_pcm_sframes_t ret;
 
   /* TODO: check the size of sbuf */
   buf = malloc(chans * sizeof(char*));
@@ -357,12 +399,6 @@ CAMLprim value ocaml_snd_pcm_writen(value handle_, value sbuf, value ofs_, value
   for(c = 0; c < chans; c++)
     free(buf[c]);
   free(buf);
-  if (ret == -EPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
-  else if (ret == -EBADFD)
-    caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
-  else if (ret == -ESTRPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
   check_for_err(ret);
 
   CAMLreturn(Val_int(ret));
@@ -377,7 +413,7 @@ CAMLprim value ocaml_snd_pcm_readn_float(value handle_, value dbuf, value ofs_,
   snd_pcm_t *handle = Pcm_handle_val(handle_);
   float **buf;
   int c, i;
-  int ret;
+  snd_pcm_sframes_t ret;
 
   /* TODO: check the size of dbuf */
   buf = malloc(chans * sizeof(float*));
@@ -395,12 +431,6 @@ CAMLprim value ocaml_snd_pcm_readn_float(value handle_, value dbuf, value ofs_,
     free(buf[c]);
   }
   free(buf);
-  if (ret == -EPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
-  else if (ret == -EBADFD)
-    caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
-  else if (ret == -ESTRPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
   check_for_err(ret);
 
   CAMLreturn(Val_int(ret));
@@ -415,7 +445,7 @@ CAMLprim value ocaml_snd_pcm_writen_float(value handle_, value fbuf, value ofs_,
   snd_pcm_t *handle = Pcm_handle_val(handle_);
   float **buf;
   int c, i;
-  int ret;
+  snd_pcm_sframes_t ret;
 
   /* TODO: check the size of fbuf */
   buf = malloc(chans * sizeof(float*));
@@ -433,12 +463,6 @@ CAMLprim value ocaml_snd_pcm_writen_float(value handle_, value fbuf, value ofs_,
   for(c = 0; c < chans; c++)
     free(buf[c]);
   free(buf);
-  if (ret == -EPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
-  else if (ret == -EBADFD)
-    caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
-  else if (ret == -ESTRPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
   check_for_err(ret);
 
   CAMLreturn(Val_int(ret));
@@ -453,7 +477,7 @@ CAMLprim value ocaml_snd_pcm_readn_float64(value handle_, value dbuf, value ofs_
   snd_pcm_t *handle = Pcm_handle_val(handle_);
   double **buf;
   int c, i;
-  int ret;
+  snd_pcm_sframes_t ret;
 
   /* TODO: check the size of fbuf */
   buf = malloc(chans * sizeof(double*));
@@ -471,12 +495,6 @@ CAMLprim value ocaml_snd_pcm_readn_float64(value handle_, value dbuf, value ofs_
     free(buf[c]);
   }
   free(buf);
-  if (ret == -EPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
-  else if (ret == -EBADFD)
-    caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
-  else if (ret == -ESTRPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
   check_for_err(ret);
 
   CAMLreturn(Val_int(ret));
@@ -491,7 +509,7 @@ CAMLprim value ocaml_snd_pcm_writen_float64(value handle_, value fbuf, value ofs
   snd_pcm_t *handle = Pcm_handle_val(handle_);
   double **buf;
   int c, i;
-  int ret;
+  snd_pcm_sframes_t ret;
 
   /* TODO: check the size of fbuf */
   buf = malloc(chans * sizeof(double*));
@@ -509,12 +527,6 @@ CAMLprim value ocaml_snd_pcm_writen_float64(value handle_, value fbuf, value ofs
   for(c = 0; c < chans; c++)
     free(buf[c]);
   free(buf);
-  if (ret == -EPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_buffer_xrun"));
-  else if (ret == -EBADFD)
-    caml_raise_constant(*caml_named_value("alsa_exn_bad_state"));
-  else if (ret == -ESTRPIPE)
-    caml_raise_constant(*caml_named_value("alsa_exn_suspended"));
   check_for_err(ret);
 
   CAMLreturn(Val_int(ret));
@@ -523,7 +535,7 @@ CAMLprim value ocaml_snd_pcm_writen_float64(value handle_, value fbuf, value ofs
 CAMLprim value ocaml_snd_pcm_start(value handle)
 {
   CAMLparam1(handle);
-  int ret;
+  snd_pcm_sframes_t ret;
 
   ret = snd_pcm_start(Pcm_handle_val(handle));
   check_for_err(ret);
@@ -726,6 +738,36 @@ CAMLprim value ocaml_snd_pcm_set_periods(value handle, value params, value perio
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value ocaml_snd_pcm_get_periods_min(value params)
+{
+  CAMLparam1(params);
+  CAMLlocal1(result);
+  int ret, dir ;
+  unsigned int periods ;
+
+  ret = snd_pcm_hw_params_get_periods_min(Hw_params_val(params), &periods, &dir);
+  check_for_err(ret);
+  result = caml_alloc_tuple(2);
+  Store_field(result,0,Val_int(periods));
+  Store_field(result,1,direction_of_int(dir));
+  CAMLreturn(result);
+}
+
+CAMLprim value ocaml_snd_pcm_get_periods_max(value params)
+{
+  CAMLparam1(params);
+  CAMLlocal1(result);
+  int ret, dir ;
+  unsigned int periods ;
+
+  ret = snd_pcm_hw_params_get_periods_max(Hw_params_val(params), &periods, &dir);
+  check_for_err(ret);
+  result = caml_alloc_tuple(2);
+  Store_field(result,0,Val_int(periods));
+  Store_field(result,1,direction_of_int(dir));
+  CAMLreturn(result);
+}
+
 CAMLprim value ocaml_snd_pcm_set_buffer_size(value handle, value params, value size)
 {
   CAMLparam3(handle, params, size);
@@ -755,6 +797,16 @@ CAMLprim value ocaml_snd_pcm_get_buffer_size_min(value params)
   CAMLreturn(Val_int(ans));
 }
 
+CAMLprim value ocaml_snd_pcm_get_buffer_size_max(value params)
+{
+  CAMLparam1(params);
+  snd_pcm_uframes_t ans;
+
+  check_for_err(snd_pcm_hw_params_get_buffer_size_max(Hw_params_val(params), &ans));
+
+  CAMLreturn(Val_int(ans));
+}
+
 static int get_access(value access)
 {
   int a = Int_val(access);
